package org.uoc.examlist.controller;

import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.uoc.examlist.view.Login;

/**
 * Clase controladora encargada de recibir las peticiones
 * relativas al login de la aplicación web.
 * 
 * @author      Roberto Calvo Mendoza
 * @version     1.0
 * @since       1.0
 */
@Controller
@RequestMapping(value = "/login")
public class LoginController {
		
	/** 
	 * Método encargado de mostrar la página con el 
	 * formulario de login.
	 *
	 * @param model     objeto modelo del framework Spring
	 * @return          nombre de la página jsp de respuesta
	 * @since           1.0
	 */		
    @RequestMapping(method = RequestMethod.GET)
    public String showLoginPage(Model model) {	    	
    	Login login = new Login();
    	model.addAttribute("login", login);
        return "login";
    }
    
    
	/** 
	 * Método encargado de validar el login
	 *
	 * @param model     objeto modelo del framework Spring
	 * @return          nombre de la página jsp de respuesta
	 * @since           1.0
	 */	    
    @RequestMapping(value = "/doLogin",method = RequestMethod.POST)
    public String doLogin(Model model) {
    	return "redirect:/examen";

    }
    
	/** 
	 * Método encargado de logout
	 * @param model     objeto modelo del framework Spring
	 * @return          nombre de la página jsp de respuesta
	 * @since           1.0
	 */	    
    @RequestMapping(value = "/logout",method = RequestMethod.GET)
    public String logout(Model model) {
    	 SecurityContextHolder.clearContext();  	 
    	
    	return "redirect:/login";

    }

}
