package org.uoc.examlist.data;

import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Set;

import javax.persistence.EntityManager;
import javax.persistence.TemporalType;
import javax.persistence.TypedQuery;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;
import org.uoc.examlist.model.AlumExam;
import org.uoc.examlist.model.AlumExamId;
import org.uoc.examlist.model.AlumExamLugar;
import org.uoc.examlist.model.ExamLugar;
import org.uoc.examlist.model.Examen;
import org.uoc.examlist.model.ProfExam;
import org.uoc.examlist.model.ProfExamLugar;
import org.uoc.examlist.model.Usuario;
import org.uoc.examlist.view.Calificacion;
import org.uoc.examlist.view.Calificaciones;
import org.uoc.examlist.view.DetalleExamen;

@Repository
@Transactional
public class ExamenDaoImpl implements ExamenDao {
	
    @Autowired
    private EntityManager em;
    
    @Override
    public List<Examen> findExamenesByIdUsuarioAlumno(int idUsuario){    	
    	TypedQuery<Examen> query = em.createQuery("SELECT e FROM AlumExam ae JOIN ae.examen e WHERE ae.alumno.usuario.id = :idUsuarioAlumno and e.fechaHoraInicio > :ahora", Examen.class);
    	query.setParameter("idUsuarioAlumno", idUsuario);
    	query.setParameter("ahora", new Date(), TemporalType.TIMESTAMP );
    	List<Examen> resultado = query.getResultList();    	
    	return resultado;    	
    }   
    
    @Override
    public List<Examen> findExamenesByIdUsuarioProfesor(int idUsuario){    	
    	TypedQuery<Examen> query = em.createQuery("SELECT e FROM ProfExam pe JOIN pe.examen e WHERE pe.profesor.usuario.id = :idUsuarioProfesor and e.fechaHoraInicio > :ahora", Examen.class);
    	query.setParameter("idUsuarioProfesor", idUsuario);
    	query.setParameter("ahora", new Date(), TemporalType.TIMESTAMP );
    	List<Examen> resultado = query.getResultList();
    	return resultado;    	
    } 
    
    @Override
    public List<Examen> findExamenesByName(String nombre){    	
    	TypedQuery<Examen> query = em.createQuery("SELECT e FROM Asignatura a JOIN a.examens e WHERE a.nombre LIKE :nombre ORDER BY e.fechaHoraInicio DESC", Examen.class);
    	query.setParameter("nombre", "%" + nombre + "%");
    	List<Examen> resultado = query.getResultList();    
    	for(Examen examen : resultado) {
    		Set<ExamLugar>  setExamLugar = examen.getExamLugars();    
    		for(ExamLugar examLugar: setExamLugar) {
    			examLugar.getLugar().getDireccion().getDireccion();
    		}
    		
    	}
    	return resultado;    	
    }
    
    @Override
    public List<Examen> findExamenesByNameIdProfesor(String nombre, int idProfesor){
    	String hQuery = "SELECT e FROM Asignatura a JOIN a.examens e JOIN e.profExams pf "
    			+ "WHERE a.nombre LIKE :nombre AND pf.profesor.id = :idProfesor "
    			+ "ORDER BY e.fechaHoraInicio DESC";
    	TypedQuery<Examen> query = em.createQuery(hQuery, Examen.class);
    	query.setParameter("nombre", "%" + nombre + "%");
    	query.setParameter("idProfesor", idProfesor);  
    	List<Examen> resultado = query.getResultList();    
    	for(Examen examen : resultado) {
    		Set<ExamLugar>  setExamLugar = examen.getExamLugars();    
    		for(ExamLugar examLugar: setExamLugar) {
    			examLugar.getLugar().getDireccion().getDireccion();
    		}
    		
    	}
    	return resultado;   
    	
    	
    	
    }
    
    @Override
    public List<Examen> findExamenesByIdAsignatura(int idAsignatura){    	
    	TypedQuery<Examen> query = em.createQuery("SELECT e FROM Asignatura a JOIN a.examens e WHERE a.id = :idAsignatura ORDER BY e.fechaHoraInicio DESC", Examen.class);
    	query.setParameter("idAsignatura", idAsignatura);
    	List<Examen> resultado = query.getResultList();    	
    	for(Examen examen : resultado) {
    		Set<ExamLugar>  setExamLugar = examen.getExamLugars();    
    		for(ExamLugar examLugar: setExamLugar) {
    			examLugar.getLugar().getDireccion().getDireccion();
    		}  		
    	}
    	return resultado;    	
    }
    
    
    @Override
    public List<Examen> findExamenesByIdAsignaturaIdProfesor(int idAsignatura, int idProfesor){   	
    	String hQuery = "SELECT e FROM Asignatura a JOIN a.examens e JOIN  e.profExams pf "
    			+ "WHERE a.id = :idAsignatura AND pf.profesor.id = :idProfesor "
    			+ "ORDER BY e.fechaHoraInicio DESC";
    	TypedQuery<Examen> query = em.createQuery(hQuery, Examen.class);
    	query.setParameter("idAsignatura", idAsignatura);
    	query.setParameter("idProfesor", idProfesor);  
    	List<Examen> resultado = query.getResultList(); 
    	for(Examen examen : resultado) {
    		Set<ExamLugar>  setExamLugar = examen.getExamLugars();    
    		for(ExamLugar examLugar: setExamLugar) {
    			examLugar.getLugar().getDireccion().getDireccion();
    		}  		
    	}
    	return resultado;   
    }
    
    @Override
    public List<AlumExam> findAlumExamByIdExamen(int idExamen){    	
    	TypedQuery<AlumExam> query = em.createQuery("SELECT ae FROM AlumExam ae JOIN ae.alumno a WHERE ae.examen.id = :idExamen ORDER BY a.apellido1 ASC", AlumExam.class);
    	//TypedQuery<AlumExam> query = em.createQuery("SELECT ae FROM AlumExam ae WHERE ae.examen.id = :idExamen ORDER BY ae.alumno.apellido1 ASC", AlumExam.class);
    	query.setParameter("idExamen", idExamen);
    	List<AlumExam> resultado = query.getResultList();    
    	for(AlumExam alumExam :  resultado) {
    		alumExam.getAlumno().getApellido1();
    	}    	
    	return resultado;    	
    }        
    
    
    @Override
    public ProfExam findProfExamByIdExamenIdProfesor(int idExamen, int idProfesor){    	
    	TypedQuery<ProfExam> query = em.createQuery("SELECT pe FROM ProfExam pe WHERE pe.examen.id = :idExamen AND pe.profesor.id = :idProfesor", ProfExam.class);
    	query.setParameter("idExamen", idExamen);
    	query.setParameter("idProfesor", idProfesor);    	
    	ProfExam resultado = query.getSingleResult();     	
    	return resultado;    	
    } 
 
    
    @Override
    public AlumExamLugar findExamenByIdUsuarioAlumnoIdExamen(int idUsuario, int idExamen){    	
    	TypedQuery<AlumExamLugar> query = em.createQuery("SELECT ael FROM AlumExamLugar ael WHERE ael.alumExam.alumno.usuario.id = :idUsuario AND ael.alumExam.examen.id= :idExamen" , AlumExamLugar.class);
    	query.setParameter("idUsuario", idUsuario);
    	query.setParameter("idExamen", idExamen);
    	AlumExamLugar alumExamLugar = query.getSingleResult();
    	alumExamLugar.getExamLugar().getExamen().getAsignatura();
    	alumExamLugar.getExamLugar().getLugar().getDireccion();
    	return alumExamLugar;    	
    } 
    
    @Override
    public ProfExamLugar findExamenByIdUsuarioProfesorIdExamen(int idUsuario, int idExamen){    	
    	TypedQuery<ProfExamLugar> query = em.createQuery("SELECT pel FROM ProfExamLugar pel WHERE pel.profExam.profesor.usuario.id = :idUsuario AND pel.profExam.examen.id= :idExamen" , ProfExamLugar.class);
    	query.setParameter("idUsuario", idUsuario);
    	query.setParameter("idExamen", idExamen);
    	ProfExamLugar profExamLugar = query.getSingleResult();
    	profExamLugar.getExamLugar().getExamen().getAsignatura();
    	profExamLugar.getExamLugar().getLugar().getDireccion();
    	return profExamLugar;    	
    }  
    
    @Override
    public List<AlumExam> findUltimasCalificacionesByIdAlumno(int idAlumno){    	
    	TypedQuery<AlumExam> query = em.createQuery("SELECT ae FROM AlumExam ae JOIN FETCH ae.examen e WHERE ae.alumno.id = :idAlumno and e.fechaHoraInicio < :ahora and e.fechaHoraInicio > :unAnioAntes", AlumExam.class);    	
    	Calendar cal = Calendar.getInstance();
    	cal.add(Calendar.YEAR, -1);
    	Date unAnioAntes = cal.getTime();    	
    	query.setParameter("idAlumno", idAlumno);
    	query.setParameter("ahora", new Date(), TemporalType.TIMESTAMP );
    	query.setParameter("unAnioAntes", unAnioAntes, TemporalType.TIMESTAMP );    	
    	List<AlumExam> resultado = query.getResultList();
    	return resultado;    	
    }
    
    
    @Override
    public ExamLugar findExamLugarByIdExamenIdLugar(int idExamen, int idLugar){    	
    	TypedQuery<ExamLugar> query = em.createQuery("SELECT el FROM ExamLugar el WHERE el.examen.id = :idExamen AND el.lugar.id= :idLugar" , ExamLugar.class);
    	query.setParameter("idExamen", idExamen);
    	query.setParameter("idLugar", idLugar);
    	ExamLugar examLugar = query.getSingleResult();
    	examLugar.getExamen().getAsignatura().getNombre();
    	examLugar.getLugar().getDireccion().getDireccion();
    	return examLugar;    	
    }
    
    @Override
	public ExamLugar updateExamLugar(DetalleExamen detalleExamen) {    	    	
    	ExamLugar examLugar = findExamLugarByIdExamenIdLugar(detalleExamen.getIdExamen(), detalleExamen.getIdLugar());
    	examLugar.getLugar().setEdificio(detalleExamen.getEdificio());
    	examLugar.getLugar().getDireccion().setDireccion(detalleExamen.getDireccion());
    	examLugar.getLugar().getDireccion().setCodPostal(detalleExamen.getCodigoPostal());  
    	examLugar.getLugar().getDireccion().setPoblacion(detalleExamen.getPoblacion()); 
       	examLugar.getLugar().getDireccion().setProvincia(detalleExamen.getProvincia()); 
       	examLugar.getLugar().setLocalizacion(detalleExamen.getAula());
       	examLugar.getLugar().setLatitud(detalleExamen.getLatitud());
       	examLugar.getLugar().setLongitud(detalleExamen.getLongitud());
       	examLugar.getLugar().setComentarios(detalleExamen.getObservaciones());
        DateFormat df = new SimpleDateFormat("dd/MM/yyyy hh:mm", new Locale("es", "ES"));
        try {	
            Date dtFechaHoraInicio =  df.parse(detalleExamen.getFecha() + " " + detalleExamen.getHora());            
            examLugar.getExamen().setFechaHoraInicio(dtFechaHoraInicio);
        } catch (ParseException pex) {
            return null;
    	}
    	examLugar = em.merge(examLugar);
    	return examLugar;
    }
    
    
    
    @Override
    public Examen findExamen(int idExamen){    	
        Examen examen = em.find(Examen.class, idExamen);
        examen.getAsignatura().getNombre();
    	return examen; 
    }
    
    
    public List<AlumExam> updateCalificaciones(Calificaciones calificaciones){
    	List<Calificacion> listCalificaciones = calificaciones.getCalificaciones();
    	BigDecimal calificacionNumerica = null;
    	for(Calificacion calificacion : listCalificaciones) {
    		calificacion.getIdAlumno();
    		calificacionNumerica = calificacion.getCalificacion();
    		//Se recupera el objeto AlumExam de BBDD
    		AlumExamId alumExamId = new AlumExamId(calificacion.getIdAlumno(), calificaciones.getIdExamen());
    		AlumExam alumExam = em.find(AlumExam.class, alumExamId);    
    		//Cambiamos la calificación y lo grabamos en BBDD
    		alumExam.setCalificacion(calificacionNumerica);
    		em.merge(alumExam);    		
    	}
    	List<AlumExam> listAlumExam =  findAlumExamByIdExamen(calificaciones.getIdExamen());
    	return listAlumExam;    	
    }
}
