package org.uoc.examlist.data;

import org.uoc.examlist.model.Profesor;
import org.uoc.examlist.model.Usuario;


/**
 * Interfaz con los accesos a base de datos para las consultas
 * relativas a los datos de usuario.
 * 
 * @author      Roberto Calvo Mendoza
 * @version     1.0
 * @since       1.0
 */
public interface UsuarioDao {
	
	
	/** 
	 * Método encargado de devolver un objeto usuario
	 * a partir de su identificador de usuario
	 *
	 * @param login 	identificador de usuario utilizado para hacer login.
	 * @return          objeto Usuario
	 * @since           1.0
	 */	
    public Usuario findUsuarioByLogin(String login);
    
    
	/** 
	 * Método encargado de actualizar el token de seguridad
	 * de un usuario en las tablas
	 *
	 * @param idUsuario	código del usuario.
	 * @param token		token de seguridad.
	 * @return          objeto Usuario
	 * @since           1.0
	 */    
    public Usuario updateToken(int idUsuario, String token);
    
    
	/** 
	 * Método encargado de devolver un objeto usuario
	 * a partir de su código de usuario
	 *
	 * @param idUsuario	código de usuario.
	 * @return          objeto Usuario
	 * @since           1.0
	 */	    
    public Usuario findUsuario(int idUsuario);
    
    
	/** 
	 * Método encargado de devolver un objeto Profesor
	 * a partir de su código de usuario
	 *
	 * @param idUsuario	código de usuario.
	 * @return          objeto Profesor
	 * @since           1.0
	 */	    
    public Profesor findProfesor(int idUsuario);

    
	/** 
	 * Método encargado de actualizar la fecha y hora
	 * del último acceso a base de datos de un usuario
	 *
	 * @param idUsuario	código de usuario.
	 * @return          objeto Usuario
	 * @since           1.0
	 */	        
    public Usuario updateAccessTime(int idUsuario);

}
