package org.uoc.examlist.utils;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Clase de utilidades genérica contiene métodos de formateo
 * 
 * @author      Roberto Calvo Mendoza
 * @version     1.0
 * @since       1.0
 */
public class Utils {
	
	public static final String EMPTY = "";
	
	/** 
	 * Método encargado formatear un objeto <code>Date</code>
	 * a formato  "dd/MM/yyyy"
	 *
	 * @param date	fecha que se quiere formatear
	 * @return      cadena con la fecha formateada a "dd/MM/yyyy"
	 * @since       1.0
	 */ 
	public static String formatFecha(Date date){
		if(date ==null) return null;
		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
		return  sdf.format(date);
	}
	
	
	/** 
	 * Método encargado formatear un objeto <code>Date</code>
	 * a formato  "hh:mm"
	 *
	 * @param date	fecha que se quiere formatear
	 * @return      cadena con la fecha formateada a "hh:mm"
	 * @since       1.0
	 */ 	
	public static String formatHora(Date date){ 
		if(date ==null) return null;
		SimpleDateFormat sdf = new SimpleDateFormat("hh:mm");
		return  sdf.format(date);		
	}	
	
	/** 
	 * Método encargado formatear un objeto <code>Date</code>
	 * a formato  "dd/MM/yyyy hh:mm"
	 *
	 * @param date	fecha que se quiere formatear
	 * @return      cadena con la fecha formateada a "dd/MM/yyyy hh:mm"
	 * @since       1.0
	 */ 
	public static String formatFechaHora(Date date){ 
		if(date ==null) return null;
		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy hh:mm");
		return  sdf.format(date);		
	}
	
	
	/** 
	 * Método encargado formatear un objeto <code>BigDecimal</code> que indica
	 * la califiacación, a formato  "#0.##"
	 *
	 * @param calif	calificación que se quiere formatear
	 * @return      cadena con la calificación formateada a "#0.##""
	 * @since       1.0
	 */ 	
	public static String formatCalificacion(BigDecimal calif){ 
		if(calif ==null) return null;
		DecimalFormat df = new DecimalFormat("#0.##");
		return df.format(calif);		
	}
	
	

}
